==> light-foundation-0.0.2.tgz: package/src/option.ts <==
import { Either, left, right } from "./either";

export type Option<T> = Some<T> | None<T>;
export function some<T>(v: T): Some<T> {
  return new Some(v);
}
export function none<T>(): None<T> {
  return new None();
}
export function opt<T>(v: T | null | undefined): Option<T> {
  if (v == null) return none();
  return some(v);
}

export class Some<T> implements OptionBase<T> {
  constructor(readonly v: T) {}
  readonly isEmpty: false = false;
  readonly isDefined: true = true;
  map<U>(f: (v: T) => U): Option<U> {
    return some(f(this.v));
  }
  flatMap<U>(f: (v: T) => Option<U>): Option<U> {
    return f(this.v);
  }
  readonly get: T = this.v;
  getOrThrow(): T {
    return this.v;
  }
  getOrElse(): T {
    return this.v;
  }
  orElse(): Option<T> {
    return this;
  }
  fold<U>(_: () => U, right: (value: T) => U): U {
    return right(this.v);
  }
  toEither<E>(): Either<E, T> {
    return right(this.v);
  }
}

export class None<T> implements OptionBase<T> {
  readonly v: undefined = undefined;
  readonly isEmpty: true = true;
  readonly isDefined: false = false;
  map<U>(): Option<U> {
    return none();
  }
  flatMap<U>(): Option<U> {
    return none();
  }
  get get(): T {
    throw new Error("None has no value");
  }
  getOrThrow(err: Error): T {
    throw err;
  }
  getOrElse(def: T): T {
    return def;
  }
  orElse(def: Option<T>): Option<T> {
    return def;
  }
  fold<U>(left: () => U): U {
    return left();
  }
  toEither<E>(err: E): Either<E, T> {
    return left(err);
  }
}

interface OptionBase<T> {
  readonly v: T | undefined;
  readonly isEmpty: boolean;
  readonly isDefined: boolean;
  map<U>(f: (v: T) => U): Option<U>;
  flatMap<U>(f: (v: T) => Option<U>): Option<U>;
  readonly get: T;
  getOrThrow(err: Error): T;
  getOrElse(def: T): T;
  orElse(def: Option<T>): Option<T>;
  fold<U>(left: () => U, right: (value: T) => U): U;
  toEither<E>(err: E): Either<E, T>;
}

==> light-foundation-0.0.4.tgz: package/src/option.ts <==
import { Either, left, right } from "./either";

export type Option<T> = Some<T> | None<T>;
export function some<T>(v: T): Some<T> {
  return new Some(v);
}
export function none<T>(): None<T> {
  return new None();
}
export function opt<T>(v: T | null | undefined): Option<T> {
  if (v == null) return none();
  return some(v);
}

export class Some<T> implements OptionBase<T> {
  constructor(readonly v: T) {}
  readonly isEmpty = false as const;
  readonly isDefined = true as const;
  map<U>(f: (v: T) => U): Option<U> {
    return some(f(this.v));
  }
  flatMap<U>(f: (v: T) => Option<U>): Option<U> {
    return f(this.v);
  }
  readonly get: T = this.v;
  getOrThrow(): T {
    return this.v;
  }
  getOrElse(): T {
    return this.v;
  }
  orElse(): Option<T> {
    return this;
  }
  fold<U>(_: () => U, right: (value: T) => U): U {
    return right(this.v);
  }
  toEither<E>(): Either<E, T> {
    return right(this.v);
  }
}

export class None<T> implements OptionBase<T> {
  readonly v = undefined;
  readonly isEmpty = true as const;
  readonly isDefined = false as const;
  map<U>(): Option<U> {
    return none();
  }
  flatMap<U>(): Option<U> {
    return none();
  }
  get get(): T {
    throw new Error("None has no value");
  }
  getOrThrow(err: Error): T {
    throw err;
  }
  getOrElse(def: T): T {
    return def;
  }
  orElse(def: Option<T>): Option<T> {
    return def;
  }
  fold<U>(left: () => U): U {
    return left();
  }
  toEither<E>(err: E): Either<E, T> {
    return left(err);
  }
}

interface OptionBase<T> {
  readonly v: T | undefined;
  readonly isEmpty: boolean;
  readonly isDefined: boolean;
  map<U>(f: (v: T) => U): Option<U>;
  flatMap<U>(f: (v: T) => Option<U>): Option<U>;
  readonly get: T;
  getOrThrow(err: Error): T;
  getOrElse(def: T): T;
  orElse(def: Option<T>): Option<T>;
  fold<U>(left: () => U, right: (value: T) => U): U;
  toEither<E>(err: E): Either<E, T>;
}
